package de.uni_marburg.powersort.benchmark;

import de.uni_marburg.powersort.data.DataEnum;
import de.uni_marburg.powersort.sort.SortEnum;
import de.uni_marburg.powersort.data.ObjectSupplier;

import java.util.EnumSet;
import java.util.concurrent.TimeUnit;

/**
 * Custom benchmark.
 */
public class Main {
    public static void main(final String[] args) {
        final EnumSet<SortEnum> sortImplementations = getSortImplementations();
        final EnumSet<DataEnum> dataEnums = getSortInputSuppliers();

        System.out.println();
        for (DataEnum dataEnum : dataEnums) {
            System.out.println("⏳ Creating object supplier " + dataEnum + " ⏳");
            ObjectSupplier objectSupplier = dataEnum.getObjectSupplier();

            for (SortEnum sortImplementation : sortImplementations) {
                if(!Filter.isFiltered(dataEnum, sortImplementation)) {
                    benchmark(objectSupplier, sortImplementation);
                }
            }

            System.out.println();
        }
    }

    static void benchmark(ObjectSupplier objectSupplier, SortEnum sortImplementation){
        System.out.print(sortImplementation);
        System.out.flush();
        Object[] sortInput = objectSupplier.getCopy();

        // TODO: JVM warmup!
        final long startNanos = System.nanoTime();
        sortImplementation.getSortImpl().sort(sortInput);
        final long stopNanos = System.nanoTime();

        final long durNanos = stopNanos - startNanos;
        final long durMillis = TimeUnit.NANOSECONDS.toMillis(durNanos);
        final String durFormatted = LongFormatter.formatUnderscore(durMillis);

        System.out.println("," + durFormatted);
    }

    static EnumSet<SortEnum> getSortImplementations() {
        return EnumSet.allOf(SortEnum.class);
    }

    /**
     * The returned ObjectSupplier objects are wrapped by DataEnum objects.
     * This way they are lazily created on their first access with DataEnum.get().
     * This saves memory if we work with large lists.
     */
    static EnumSet<DataEnum> getSortInputSuppliers() {
        return EnumSet.allOf(DataEnum.class);
    }
}
